unit FormMain;

{$I Global_Conditionals.INC}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, Menus, StdCtrls, ImgList, Buttons, Spin, ShellAPI,
  {$IfDef HELIOS_APP}Helios_Engine,{$Else}Helios_Basic,{$EndIf} SynEdit, SynEditHighlighter, SynUniHighlighter, strutils,
  Helios_Shader_System, jpeg;

Const
APPLICATION_TITLE = 'Project Helios: Shader Editor 2';
APPLICATION_VER = '1.0';
APPLICATION_BY = 'Stucuk';

type
  TFrmMain = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Panel1: TPanel;
    StatusBar1: TStatusBar;
    MainView: TPanel;
    Help1: TMenuItem;
    About1: TMenuItem;
    ools1: TMenuItem;
    ScreenShot1: TMenuItem;
    BackgroundColour2: TMenuItem;
    N2: TMenuItem;
    Options1: TMenuItem;
    Disable3DView1: TMenuItem;
    TopBarImageHolder: TImage;
    Label3: TLabel;
    Label21: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Panel2: TPanel;
    Panel3: TPanel;
    ShaderList: TComboBox;
    ColorDialog: TColorDialog;
    SynUniSyn: TSynUniSyn;
    OpenDialog: TOpenDialog;
    Open1: TMenuItem;
    N1: TMenuItem;
    ModBox: TComboBox;
    Label1: TLabel;
    Bevel1: TBevel;
    SynEdit1: TSynEdit;
    Panel4: TPanel;
    Button1: TButton;
    SaveShader1: TMenuItem;
    SaveDialog: TSaveDialog;
    ShaderSpin: TSpinButton;
    Search1: TMenuItem;
    N3: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Idle(Sender: TObject; var Done: Boolean);
    Procedure SetCaption(Filename : String);
    Procedure SetIsEditable;
    procedure MainViewResize(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure ScreenShot1Click(Sender: TObject);
    procedure BackgroundColour2Click(Sender: TObject);
    procedure Disable3DView1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Open1Click(Sender: TObject);
    procedure ShaderListChange(Sender: TObject);
    procedure SynEdit1Change(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    Procedure GetModList;
    Procedure SaveShader(Filename : String);
    procedure SaveShader1Click(Sender: TObject);
    procedure ShaderSpinDownClick(Sender: TObject);
    procedure ShaderSpinUpClick(Sender: TObject);
    procedure ModBoxChange(Sender: TObject);
    procedure Search1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    ShaderLoaded : Boolean;
    Filename : String;
    Procedure BuildShaderList;
  end;

var
  FrmMain: TFrmMain;
  ShaderManager : TShaderManager;

implementation

uses ThreeDView_Engine, FormAboutNew, FormLoadShader, FormFindTex;

{$R *.dfm}

Function FloatToStr2(Float : Extended) : String; // Makes numbers more understanding
begin

Result := FloatToStrF(Float,ffNumber,7,20);

if Copy(Result,length(Result),1) = '0' then
repeat
Result := Copy(Result,1,length(Result)-1);
until Copy(Result,length(Result),1) <> '0';

if Copy(Result,length(Result),1) = '.' then
Result := Result + '0';

end;

procedure TFrmMain.Exit1Click(Sender: TObject);
begin
Close;
end;

procedure TFrmMain.Idle(Sender: TObject; var Done: Boolean);
begin
if (not MVEnabled) then exit;

DrawFrames;

Done := false;
end;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
SetCaption('');
//SynEdit1.DoubleBuffered := True;
If fileexists(ExtractFileDir(ParamStr((0))) + '\Q3Shader.HGL') then
SynUniSyn.LoadFromFile(ExtractFileDir(ParamStr((0))) + '\Q3Shader.HGL')
else
begin
MessageBox(0,'Q3Shader.hgl not found','Missing File',0);
Close;
end;
MainViewWidth := MainView.Width;
MainViewHeight := MainView.Height;

Application.OnIdle := Idle;
Init(MainView.Handle);

SetIsEditable;

Form_StatusBar := @StatusBar1;
GetModList;

ShaderManager := TShaderManager.Create('');
end;

Procedure TFrmMain.SetCaption(Filename : String);
begin

If Filename <> '' then
Caption := ' ' + APPLICATION_TITLE + ' v'+APPLICATION_VER + ' [' +Extractfilename(Filename) + ']'
else
Caption := ' ' + APPLICATION_TITLE + ' v'+APPLICATION_VER;

end;

Procedure TFrmMain.SetIsEditable;
var
Value : Boolean;
begin
Value := ShaderLoaded;

ools1.Visible := Value;
Options1.Visible := Value;
ShaderList.Enabled := Value;
ModBox.Enabled := Value;
Button1.Enabled := Value;
SynEdit1.Enabled := Value;
SaveShader1.Enabled := Value;
ShaderSpin.Enabled := Value;

if Value then
MainView.Cursor := crCross
else
MainView.Cursor := crDefault;

end;

procedure TFrmMain.MainViewResize(Sender: TObject);
begin
If oglloaded then
ResizeMainView(MainView.Width,MainView.Height);
end;

procedure TFrmMain.About1Click(Sender: TObject);
var
  frm: TFrmAbout_New;
begin
  frm:=TFrmAbout_New.Create(Self);
  frm.Visible:=False;
  Frm.Image1.Picture := TopBarImageHolder.Picture;
  frm.Label2.Caption := APPLICATION_TITLE + ' v' + APPLICATION_VER;
  frm.label1.Caption := frm.label1.Caption + APPLICATION_BY;
  frm.ShowModal;
  frm.Close;
  frm.Free;
end;

procedure TFrmMain.ScreenShot1Click(Sender: TObject);
begin
{$IfDef HELIOS_APP}Take_ScreenShot;{$EndIf}
end;

procedure TFrmMain.BackgroundColour2Click(Sender: TObject);
begin
ColorDialog.Color := TVector3fToTColor(BGColor);
if ColorDialog.Execute then
BGColor := TColorToTVector3f(ColorDialog.Color);
end;

procedure TFrmMain.Disable3DView1Click(Sender: TObject);
begin
MVEnabled := not MVEnabled;
Disable3DView1.Checked := not MVEnabled;

if not MVEnabled then // no point refreshing if opengl is on
MainView.Refresh;
end;

procedure TFrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
{$IfDef HELIOS_APP}DestroyHelios;{$EndIf}
end;

function RemoveComment(txt : string) : string;
var p : integer;
    tmp : string;
begin
  txt := Trim(Txt);
  txt := StringReplace(txt, #9,  ' ', [rfReplaceAll]);

  tmp := txt;
  p := Pos('//', txt);
  if p > 0 then
    tmp := Trim(Copy(txt, 1, p-1));
  p := Pos('#', txt);
  if p > 0 then
    tmp := Trim(Copy(txt, 1, p-1));
  result := Trim(tmp);
end;

function ConvertPath(p : string) : string;
begin
  result := StringReplace(p, '/', '\', [rfReplaceAll]);
end;

procedure TFrmMain.Open1Click(Sender: TObject);
var
  frm: TFrmLoadShader;
begin
  frm:=TFrmLoadShader.Create(Self);
  frm.Visible:=False;
  Frm.Image1.Picture := TopBarImageHolder.Picture;
  frm.ModBox.Items.Text  := ModBox.Items.Text;
  frm.ModBox.ItemIndex := ModBox.ItemIndex;
  frm.CreateShaderlist;
  frm.ShowModal;
  frm.Close;

If frm.o then
begin

If ModBox.ItemIndex <> frm.ModBox.ItemIndex then
begin
ModBox.ItemIndex := frm.ModBox.ItemIndex;
ModBoxChange(Sender);
end;

If ShaderManager.ShaderSrc_No > 0 then
ShaderLoaded := True
else
ShaderLoaded := False;
If ShaderLoaded then
begin
SetCaption(frm.FileName);
Filename := frm.FileName;
BuildShaderList;
SetIsEditable;
end;
end;

frm.Free;

end;

Procedure TFrmMain.BuildShaderList;
var
X : integer;
Sender : TObject;
ShaderFile : String;
begin

ShaderList.Items.Clear;

ShaderFile := ansilowercase(Copy(extractfilename(Filename),1,length(extractfilename(Filename))-length(extractfileext(Filename))));

If not ShaderLoaded then exit;

If ShaderManager.ShaderSrc_no > 0 then
for x := 0 to ShaderManager.ShaderSrc_no-1 do
If ShaderManager.ShaderSrc[x].Shader = ShaderFile then
ShaderList.Items.Add(ShaderManager.ShaderSrc[x].Name);

ShaderList.ItemIndex := 0;
ShaderListChange(Sender);
end;

procedure TFrmMain.ShaderListChange(Sender: TObject);
begin
If ShaderList.Items.Count < 1 then exit;

SynEdit1.Text := ShaderManager.ShaderSrc[ShaderManager.FindShaderSrc(ShaderList.Items.Strings[ShaderList.itemindex])].Data;
Button1.Click;
end;

procedure TFrmMain.SynEdit1Change(Sender: TObject);
begin
if ShaderList.Items.Count < 1 then exit;

ShaderManager.ShaderSrc[ShaderManager.FindShaderSrc(ShaderList.Items.Strings[ShaderList.itemindex])].Data := SynEdit1.Text;
end;

procedure TFrmMain.Button1Click(Sender: TObject);
begin
ShaderManager.Clear;
ShaderManager.ReadShadersFromString('SHADERTEST',SynEdit1.Text);
ShaderManager.ProcessShaderTextures;
If ShaderManager.IndexOf('SHADERTEST') < 0 then
ShowMessage('Error');
end;

Procedure TFrmMain.GetModList;
var
f: TSearchRec;
x : integer;
begin

ModBox.Items.Clear;

if FindFirst(extractfiledir(paramstr(0)) + '\*',faDirectory,f) = 0 then repeat

if DirectoryExists(extractfiledir(paramstr(0)) + '\' + f.Name) then
if fileexists(extractfiledir(paramstr(0)) + '\' + f.Name +'\liblist.gam') then
ModBox.Items.Add(f.Name);

until FindNext(f) <> 0;
FindClose(f);

ModBox.ItemIndex := 0;

if ModBox.Items.Count > 0 then
for x := 0 to ModBox.Items.Count -1 do
if ansilowercase(ModBox.Items.Strings[x]) = 'main' then
begin
ModBox.ItemIndex := x;
break;
end;

end;

Procedure TFrmMain.SaveShader(Filename : String);
var
S : TStringlist;
x : integer;
D,ShaderFile : String;
begin

ShaderFile := ansilowercase(Copy(extractfilename(FrmMain.Filename),1,length(extractfilename(FrmMain.Filename))-length(extractfileext(FrmMain.Filename))));

S := TStringList.Create;

For x := 0 to ShaderManager.ShaderSrc_no-1 do
If ShaderFile = ShaderManager.ShaderSrc[x].Shader then
begin
ShaderManager.ShaderSrc[x].Shader := ShaderFile;
D := ShaderManager.ShaderSrc[x].Name + #13 + '{' + #13 + ShaderManager.ShaderSrc[x].Data + #13 + '}';
if S.Text = '' then
S.Text := D
else
S.Text := S.Text + #13#13+ D;
end;

S.SaveToFile(Filename);
S.Free;
end;

procedure TFrmMain.SaveShader1Click(Sender: TObject);
begin
if Filename <> '' then
SaveDialog.Filename := StringReplace(StringReplace(Filename, '\\',  '\', [rfReplaceAll]), '//',  '/', [rfReplaceAll]);

if SaveDialog.Execute then
SaveShader(SaveDialog.Filename);
end;

procedure TFrmMain.ShaderSpinDownClick(Sender: TObject);
begin
If ShaderList.ItemIndex > 0 then
ShaderList.ItemIndex := ShaderList.ItemIndex - 1
else
ShaderList.ItemIndex := ShaderList.Items.Count-1;

ShaderListChange(Sender);
end;

procedure TFrmMain.ShaderSpinUpClick(Sender: TObject);
begin
If ShaderList.Items.Count-1 > ShaderList.ItemIndex then
ShaderList.ItemIndex := ShaderList.ItemIndex + 1
else
ShaderList.ItemIndex := 0;

ShaderListChange(Sender);
end;

procedure TFrmMain.ModBoxChange(Sender: TObject);
begin
ChangeModFolder(ModBox.Text);
ShaderManager.Destroy;
ShaderManager := TShaderManager.Create('');
end;

procedure TFrmMain.Search1Click(Sender: TObject);
var
  frm: TFrmFindTex;
  ID : Integer;
begin
  frm:=TFrmFindTex.Create(Self);
  frm.Visible:=False;

  frm.ShowModal;

  If frm.O then
  begin
  ID := ShaderManager.FindShaderSrc(frm.Edit1.Text);
  If ID > -1 then
  MessageBox(0,Pchar('Texture: ' + #13 + frm.Edit1.Text + #13#13 + 'ShaderFile: ' +#13 +ShaderManager.ShaderSrc[ID].Shader),'Find Texture',0)
  else
  MessageBox(0,Pchar('Texture: ' + #13 + frm.Edit1.Text + #13#13 + 'Not Found'),'Find Texture',0);
  end;

  frm.Close;
  frm.Free;
end;

end.
