unit Helios_Basic;
{
Description: Used to emulate basic helios things.
}

interface

Uses Windows,Math3d,OpenGl15,Helios_Timer;

//Helios_Global.Pas
Const
HELIOS_MODFOLDER = 'Main';
DEPTH_OF_VIEW = 4000;
FOV = 45;

Var
GAME_FOLDER,MOD_FOLDER,BaseDir : String;
SCREEN_WIDTH,SCREEN_HEIGHT : Integer;
lastError : Cardinal;
lastErrorString : String;
h_DC   : HDC;
h_RC   : HGLRC;
h_Wnd  : HWND;
UseCameraByMouse : Boolean; // Not Used

//Helios_Types.Pas
type TBSPVertex = record
  Position      : TVector3f;             // (x, y, z) position.
  TextureCoord  : TVector2f;             // (u, v) texture coordinate
  LightmapCoord : TVector2f;             // (u, v) lightmap coordinate
  Normal        : TVector3f;             // (x, y, z) normal vector
  Color         : array[0..3] of Byte    // RGBA color for the vertex
end;

TRGBA = array[0..3] of GLUbyte;

//Q3BSP.Pas
var fogBuffer : array of single;
    inputBuffer : array[0..16383] of TBSPVertex;
    texCoordBuffer :  array[0..16383] of TVector2f;
    colorBuffer : array[0..16383] of TRGBA;

//Helios_Global.Pas
function StrToFloat(s : string) : extended;

//Helios_Engine.Pas
Procedure ChangeModFolder(ModName : string);
Procedure InitiateHelios(GameFolder,ModFolder, Params : String);
procedure glResizeWnd(Width, Height : Integer);

//Helios_FileSystem.Pas
Function DoesFileExist(Filename : String) : String;

//Q3BSP.Pas
procedure SetDefaults;

implementation

Uses SysUtils;

//Helios_Global.Pas
function StrToFloat(s : string) : extended;
begin
  try
   // s := StringReplace(s, '.', ',', [rfReplaceAll]);
    result := sysutils.StrToFloat(s);
  except
  Try
    s := StringReplace(s, '.', ',', [rfReplaceAll]);
    result := sysutils.StrToFloat(s);
  except
    result := 0.0;
  end;
  end;
end;

//Helios_Engine.Pas
Function DoesModFolderExist(ModName : string) : Boolean;
begin
Result := fileexists(GAME_FOLDER + '\' + ModName + '\liblist.gam');
end;


Procedure ChangeModFolder(ModName : string);
begin
  if DoesModFolderExist(ModName) then
  MOD_FOLDER := ModName;
end;

Procedure InitiateHelios(GameFolder,ModFolder, Params : String);
begin
GAME_FOLDER := GameFolder;
MOD_FOLDER := ModFolder;
gTimer := THiResTimer.Create;
end;

procedure glResizeWnd(Width, Height : Integer);
begin
SCREEN_WIDTH := Width;
SCREEN_HEIGHT := Height;

  if (Height = 0) then                // prevent divide by zero exception
    Height := 1;
  glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
  glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
  glLoadIdentity();                   // Reset View
//  gluPerspective(FOV, Width/Height, 1.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
  gluPerspective(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
//  PerspectiveGL(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);
//   glFrustum(-40.0, 40.0, -20.704225, 20.704225, 4.0, 4096.0);  //GLQuake Mode
  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
  glLoadIdentity();                   // Reset View
end;

//Helios_FileSystem.Pas
Function DoesFileExist(Filename : String) : String; // If Exists filename is given as result
begin

if MOD_FOLDER <> '' then
if fileexists(GAME_FOLDER+'\'+MOD_FOLDER+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+MOD_FOLDER+'\'+Filename;
Exit;
end;

if fileexists(GAME_FOLDER+'\'+HELIOS_MODFOLDER+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+HELIOS_MODFOLDER+'\'+Filename;
Exit;
end;

If Basedir <> '' then
if fileexists(GAME_FOLDER+'\'+Basedir+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+Basedir+'\'+Filename;
Exit;
end;

if fileexists(Filename) then
begin
Result := Filename;
Exit;
end;

Result := '';

end;

//Q3BSP.Pas
procedure SetDefaults;
begin
  //glDepthFunc(GL_LEQUAL);  //
  //glEnable(GL_DEPTH_TEST); //
  glDisable(GL_ALPHA_TEST);
  glDepthMask(true);
  glDisable(GL_BLEND);
  {glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);}

  glDisable(GL_COLOR);  //
{  if CG_MultiTexture then begin //
    glActiveTexture(GL_TEXTURE1);
    glDisableClientState(GL_COLOR_ARRAY);
    glActiveTexture(GL_TEXTURE0);
  end;   }
  glDisableClientState(GL_COLOR_ARRAY);   //
  glColor3ub(255,255,255);  //
  glCullFace(GL_FRONT);
  //glEnable(GL_CULL_FACE);  //
end;

end.
